/*
 *
 * Hunter McCurry
 *
 *
 *
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>
//#include "rt/RtMidi.h"
#include "ChordList.h"
#include <iostream>
#include <sys/time.h>

// note: for mac only
#include <GLUT/glut.h>

#include "BeeThree.h"
#include "PercFlut.h"
#include "StifKarp.h"
#include "BandedWG.h"
#include "FM.h"
#include "Rhodey.h"
#include "Moog.h"
#include "RtAudio.h"
#include "Messager.h"
#include "Voicer.h"

using std::min;

using namespace stk;


// width and height of the window
GLsizei g_width = 1280;
GLsizei g_height = 800;


// playing field things
float default_height = 5;
float shrink_rate = .3;

float field_width = 12.0;
float field_height = default_height;

float leftEdge = -field_width / 2;
float rightEdge = field_width / 2;
float bottomEdge = - field_height / 2;
float topEdge = field_height / 2;

// sound things
bool started = false;
std::vector <long> paddleNotesPlaying (128);
int ballPolyphony = 1;


// paddle things
int keyboardPolyphony = 10;

float horizIncrement = (field_width/2)/13.0;
float paddle_height = .22;
float paddle_space = .08;
int paddlePosition = 0;

float currentPositionLeft;
float currentPositionRight;

bool real_paddle = false;
bool passed_paddle = false;



// ball things
float ballDefaultSpeed = 1;
float ballSpeed = 1.5 * ballDefaultSpeed;
float ball_accel = .1;

float ball_radius = .2;
float ball_x = 0;
float ball_y = 0;
float ball_vx = 1.0;
float ball_vy = 1.0;

float future_ball_radius = .2;
float future_ball_x;
float future_ball_y;
float future_ball_vx;
float future_ball_vy;

float future_ball_left;
float future_ball_right;
float future_ball_bottom;
float future_ball_top;

float snitch_x;
float snitch_y;
float snitch_radius = .4;
GLfloat snitch_rotation = 0;
int score = 0;

bool future_found = false;


// clock things
timeval current_time;
timeval last_time;
double time_counter;
double time_diff = .001;
double last_ball_note = 0;





//-----------------------------------------------------------------------------
// function prototypes
//-----------------------------------------------------------------------------
void idleFunc( );
void displayFunc( );
void reshapeFunc( GLsizei width, GLsizei height );
void keyboardFunc( unsigned char, int, int );
void mouseFunc( int button, int state, int x, int y );
void getNotez(  double deltatime, std::vector< unsigned char > *message, void *userData );
void resetBall( );
void findFuturePath( );
void resizeField( float w, float h );
void lookupPaddles( );
void drawFrame( );
void drawBall( float xPos, float yPos, float rad, bool real, bool real );
void randomizeSnitch( );
void playBallNote( );



// The TickData structure holds all the class instances and data that
// are shared by the various processing functions.
struct TickData {
	Voicer voicer;
	int counter;
	long myId;
	bool done;
	
	// Default constructor.
	TickData()
    : counter(0), done( false ) {}
};


TickData data;
TickData * dataRef = &data;

// This tick() function handles sample computation and scheduling of
// control updates.  
int tick( void *outputBuffer, void *inputBuffer, unsigned int nBufferFrames,
         double streamTime, RtAudioStreamStatus status, void *dataPointer )
{
	TickData *data = (TickData *) dataPointer;
	register StkFloat *samples = (StkFloat *) outputBuffer;
	int counter, nTicks = (int) nBufferFrames;
			
	for ( int i=0; i<nBufferFrames; i++ ) {
		*samples++ = data->voicer.tick() * 2;
		*samples++ = data->voicer.tick() * 2;
	}
	
	return 0;
}



// The midi callback function
void getNotez( double deltatime, std::vector< unsigned char > *message, void *userData )
{	
	unsigned int nBytes = message->size();
	
	if (nBytes == 3) {
		if ((int)message->at(2) != 0 && (int)message->at(0) == 144) { // only look at note on messages with a non-zero velocity
			int midiPitch = (int)message->at(1);
			int midiVelocity = (int)message->at(2);
			
			int notePosition = 0;
			int more = 1;
			paddleNotesPlaying[midiPitch] = dataRef->voicer.noteOn( midiPitch, 20 );
			
			
		}
		else if ((int)message->at(2) == 0  || (int)message->at(0) == 144 ) {
			int midiPitch = (int)message->at(1);
			
			dataRef->voicer.noteOff( paddleNotesPlaying[midiPitch], 64);
			paddleNotesPlaying[midiPitch] = 0;
		}
		
	}
}




// entry point
int main( int argc, char ** argv )
{	
	// Set up RtAudio Junk
	//--------------------
	// Set the global sample rate and rawwave path before creating class instances.
	Stk::setSampleRate( 44100.0 );
	Stk::setRawwavePath( "stk/rawwaves/" );	
	RtAudio dac;
	Instrmnt *keyInstrument[keyboardPolyphony];
	Instrmnt *hitInstrument[3];
	Instrmnt *looseInstrument[3];
	Instrmnt *ballInstrument[ballPolyphony];
	Instrmnt *snitchInstrument[1];
	
	for ( int i=0; i<keyboardPolyphony; i++ ) keyInstrument[i] = 0;
	for ( int i=0; i<3; i++ ) hitInstrument[i] = 0;
	for ( int i=0; i<3; i++ ) looseInstrument[i] = 0;
	for ( int i=0; i<ballPolyphony; i++ ) ballInstrument[i] = 0;
	for ( int i=0; i<1; i++ ) ballInstrument[i] = 0;
	
	// Figure out how many bytes in an StkFloat and setup the RtAudio stream.
	RtAudio::StreamParameters parameters;
	parameters.deviceId = dac.getDefaultOutputDevice();
	parameters.nChannels = 2;
	RtAudioFormat format = ( sizeof(StkFloat) == 8 ) ? RTAUDIO_FLOAT64 : RTAUDIO_FLOAT32;
	unsigned int bufferFrames = RT_BUFFER_SIZE;
	
	
	// Set up RtMidi Junk	
	//-------------------
	RtMidiIn *midiin = 0;
	// RtMidiIn constructor
	try {
		midiin = new RtMidiIn();
	}
	catch ( RtError &error ) {
		error.printMessage();
		exit( EXIT_FAILURE );
	}
	// Check available ports vs. specified.
	unsigned int nPorts = midiin->getPortCount();
	
	
	try {
		dac.openStream( &parameters, NULL, format, (unsigned int)Stk::sampleRate(), &bufferFrames, &tick, (void *)&data );
	}
	catch ( RtError &error ) {
		error.printMessage();
		goto cleanup;
	}
	
	
	// Define and key-press instruments
	try {
		for ( int i=0; i<keyboardPolyphony; i++ )
			keyInstrument[i] = new PercFlut();
	}
	catch ( StkError & ) {
		goto cleanup;
	}
	
	// Define paddle hit instruments
	try {
		for ( int i=0; i<3; i++ )
			hitInstrument[i] = new Rhodey();
	}
	catch ( StkError & ) {
		goto cleanup;
	}
	
	// Define the loose instruments
	try {
		for ( int i=0; i<3; i++ )
			looseInstrument[i] = new Moog();
	}
	catch ( StkError & ) {
		goto cleanup;
	}
	// Define the ball instruments
	try {
		for ( int i=0; i<3; i++ )
			ballInstrument[i] = new BandedWG();
	}
	catch ( StkError & ) {
		goto cleanup;
	}
	// Define the ball instruments
	try {
		for ( int i=0; i<1; i++ )
			snitchInstrument[i] = new StifKarp(40);
	}
	catch ( StkError & ) {
		goto cleanup;
	}
	
	// "Add" the instruments to the voicer.
	for ( int i=0; i<keyboardPolyphony; i++ )
		data.voicer.addInstrument( keyInstrument[i], 0 );
	for ( int i=0; i<3; i++ )
		data.voicer.addInstrument( hitInstrument[i], 1 );
	for ( int i=0; i<3; i++ )
		data.voicer.addInstrument( looseInstrument[i], 2 );
	for ( int i=0; i<ballPolyphony; i++ )
		data.voicer.addInstrument( ballInstrument[i], 3 );
	for ( int i=0; i<1; i++ )
		data.voicer.addInstrument( snitchInstrument[i], 4 );
	
	// Start Audio Stream
	try {
		dac.startStream();
	}
	catch ( RtError &error ) {
		error.printMessage();
		goto cleanup;
	}
	
	
	// Start Midi Callback
	if ( nPorts > 0) {
		try {
			midiin->openPort( nPorts - 1 );
		}
		catch ( RtError &error ) {
			error.printMessage();
			goto cleanup;
		}
		midiin->setCallback( &getNotez , (void *) NULL );
		std::cout << "\n\nMidi keyboard found, play away!" << std::endl;
		std::cout << "press 'q' to end program.\n" << std::endl;
	}
	else {
		std::cerr << "I can't seem to find any midi ports...\n" << std::endl;
		goto cleanup;
	}
	
	// Give the ball an initial location and velocity vector
	resizeField( field_width, default_height );
	randomizeSnitch( );
	resetBall( );
	
	
    // initialize GLUT
    glutInit( &argc, argv );
    // double buffer, use rgb color, enable depth buffer
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
    // initialize the window size
    glutInitWindowSize( g_width, g_height );
    // set the window position
    glutInitWindowPosition( 100, 100 );
    // create the window
    glutCreateWindow( "Piano Pong" );
	
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    // set the idle function - called when idle
    glutIdleFunc( idleFunc );
    // set the display function - called when redrawing
    glutDisplayFunc( displayFunc );
    // set the reshape function - called when client area changes
    glutReshapeFunc( reshapeFunc );
    // set the keyboard function - called on keyboard events
    glutKeyboardFunc( keyboardFunc );
    // set the mouse function - called on mouse stuff
    glutMouseFunc( mouseFunc );
	
	
    // let GLUT handle the current thread from here
    glutMainLoop();
	
    // Shut down the callback and output stream.  
	try {
		dac.closeStream();
	}
	catch ( RtError &error ) {
		error.printMessage();
	}
	
cleanup:
	delete midiin;
	
	//glutLeaveGameMode();
	for ( int i=0; i<keyboardPolyphony; i++ ) delete keyInstrument[i];
	for ( int i=0; i<3; i++ ) delete hitInstrument[i];
	for ( int i=0; i<3; i++ ) delete looseInstrument[i];
	for ( int i=0; i<ballPolyphony; i++ ) delete ballInstrument[i];
	for ( int i=0; i<1; i++ ) delete ballInstrument[i];
	
	return 0;
}


//-----------------------------------------------------------------------------
// Name: reshapeFunc( )
// Desc: called when window size changes
//-----------------------------------------------------------------------------
void reshapeFunc( GLsizei w, GLsizei h )
{
	// save the new window size
    // g_width = w; g_height = h;
    // map the view port to the client area
    glViewport( 0, 0, g_width, g_height );
    // set the matrix mode to project
    glMatrixMode( GL_PROJECTION );
    // load the identity matrix
    glLoadIdentity( );
    // create the viewing frustum
    gluPerspective( 45.0, (GLfloat) g_width / (GLfloat) g_height, 1.0, 300.0 );
    // set the matrix mode to modelview
    glMatrixMode( GL_MODELVIEW );
    // load the identity matrix
    glLoadIdentity( );
    // position the view point
    gluLookAt( 0.0f, 0.0f, 10.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );
	glutPostRedisplay();
}



//-----------------------------------------------------------------------------
// Name: keyboardFunc( )
// Desc: key event
//-----------------------------------------------------------------------------
void keyboardFunc( unsigned char key, int x, int y )
{
    switch( key )
    {
        case 'Q':
        case 'q':
            exit(1);
            break;
    }
    
    glutPostRedisplay( );
}




//-----------------------------------------------------------------------------
// Name: mouseFunc( )
// Desc: handles mouse stuff
//-----------------------------------------------------------------------------
void mouseFunc( int button, int state, int x, int y )
{
    if( button == GLUT_LEFT_BUTTON )
    {
        // when left mouse button is down
        if( state == GLUT_DOWN )
        {
        }
        else
        {
        }
    }
    else if ( button == GLUT_RIGHT_BUTTON )
    {
        // when right mouse button down
        if( state == GLUT_DOWN )
        {
        }
        else
        {
        }
    }
    else
    {
    }
    
    glutPostRedisplay( );
}



//-----------------------------------------------------------------------------
// Name: idleFunc( )
// Desc: callback from GLUT
//-----------------------------------------------------------------------------
void idleFunc( )
{
	// Update ball state based on the time
	gettimeofday(&current_time, NULL);

	time_counter = current_time.tv_sec + (current_time.tv_usec / 1000000.0);
	double tmp2 = last_time.tv_sec + (last_time.tv_usec / 1000000.0);
	
	time_diff = time_counter - tmp2;
		
	if (time_counter > last_ball_note) {
		playBallNote( );
		last_ball_note = time_counter + 1.0 / (2 * ballSpeed);
	}
	
	last_time = current_time;
	
	ball_x = ball_x +  time_diff *  ballSpeed * ball_vx / 1.0;
	ball_y = ball_y + time_diff *  ballSpeed * ball_vy / 1.0;
	
	
	float ball_left = ball_x - ball_radius;
	float ball_right = ball_x + ball_radius;
	float ball_bottom = ball_y - ball_radius;
	float ball_top = ball_y + ball_radius;
	
	
	// Snitch Collection!
	float x_dist = fabs( ball_x - snitch_x );
	float y_dist = fabs( ball_y - snitch_y );
	float total_dist = sqrt( pow(x_dist, 2) + pow(y_dist, 2) );
	
	if ( ( ball_radius + snitch_radius) > total_dist ) { // got it!
		if ( field_height < default_height) {
			resizeField( field_width, default_height );
		}

		dataRef->voicer.noteOn( 80, 80, 4 );
		score++;
		ballSpeed += ball_accel;
		
		future_ball_x = ball_x;
		future_ball_y = ball_y;
		future_ball_vx = ball_vx;
		future_ball_vy = ball_vy;
		randomizeSnitch( );
		
		findFuturePath( );
	}
	
	
	// Collisions!
	// x
	if ( ball_right	> rightEdge ) {
		ball_x = rightEdge - (ball_right - rightEdge) - ball_radius;
		ball_vx = -ball_vx;
	}
	else if ( ball_left < leftEdge ){
		ball_x = leftEdge + (leftEdge - ball_left) + ball_radius;
		ball_vx = -ball_vx;
	}
	// y
	if ( ball_top > topEdge ) {
		ball_y = topEdge - (ball_top - topEdge) - ball_radius;
		ball_vy = -ball_vy;
	}
	else if ( ball_bottom < (bottomEdge + paddle_height)) {  // within paddle range
		if ( !passed_paddle ) {    // just look at the *first* time it's within range
			if ( real_paddle ) {   // there is a solid paddle

				float paddle_left = currentPositionLeft + paddle_space - .15;
				float paddle_right = currentPositionRight - paddle_space + .15;
				float paddle_width = paddle_right - paddle_left;
				float paddle_top = bottomEdge + paddle_height;
								
				if ( ball_x > paddle_left && ball_x < paddle_right ) { // paddle hit!!

					int note1 = NoteSequence[(paddlePosition + 2) % 24] + 48;
					dataRef->voicer.noteOn( note1, 10, 1 );
					int note2 = NoteSequence[(paddlePosition + 3) % 24] + 48;
					dataRef->voicer.noteOn( note2, 10, 1 );
					int note3 = NoteSequence[(paddlePosition + 4) % 24] + 48;
					dataRef->voicer.noteOn( note3, 10, 1 );
					
					resizeField( field_width, field_height - shrink_rate );
					
					paddle_top = bottomEdge + paddle_height;
					ball_y = paddle_top + (paddle_top - ball_bottom) + ball_radius;
					
					float ball_pct_dist = ( ball_x - paddle_left ) / paddle_width;
					float theta =  PI/8 + (3*PI/4 - ( ball_pct_dist * (3*PI/4) ) );
				
					ball_vy = 1.5 * ballSpeed * sin(theta);
					ball_vx = 1.5 * ballSpeed * cos(theta);
					
					
					// initialize future ball bounds
					future_ball_x = ball_x;
					future_ball_y = ball_y;
					future_ball_vx = ball_vx;
					future_ball_vy = ball_vy;
					
					findFuturePath( );
					
				}
				else {		// there will be a score!
					passed_paddle = true;
				}
				
			}
			else {			// there will be a score!
				passed_paddle = true;
			}
		}
		
		if ( ball_bottom < bottomEdge ){  // goal!!!!!!!!!
			if (started) {
				long thing1 = dataRef->voicer.noteOn( 12, 60, 2 );
				long thing2 = dataRef->voicer.noteOn( 15, 60, 2 );
				long thing3 = dataRef->voicer.noteOn( 18, 60, 2 );
				usleep(1000000);
				dataRef->voicer.noteOff( thing1, 64);
				dataRef->voicer.noteOff( thing2, 64);
				dataRef->voicer.noteOff( thing3, 64);
			}
			
			resizeField( field_width, default_height );
			
			randomizeSnitch( );
			resetBall( );
			started = true;
		}
	}
	
	
	// render the scene
    glutPostRedisplay( );
}

//-----------------------------------------------------------------------------
// Name: displayFunc( )
// Desc: callback function invoked to draw the client area
//-----------------------------------------------------------------------------
void displayFunc( )
{
    // clear the color and depth buffers
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    
    // line width
    glLineWidth( 3.0 );
    // step through and plot the waveform
    GLfloat x = -5;
    // increment
    GLfloat xinc = .1;
    
    // push the matrix
    glPushMatrix();
	
    // go
    glBegin( GL_LINE_STRIP );
	
    // done
    glEnd();
    
    // pop
    glPopMatrix();
	
	
	lookupPaddles();
	drawFrame();
	
	drawBall( ball_x, ball_y, ball_radius, true, false );
	drawBall( future_ball_x, future_ball_y, future_ball_radius, false, false );
	drawBall( snitch_x, snitch_y, snitch_radius, true, true );
	
	
    // flush!
    glFlush( );
    // swap the double buffer
    glutSwapBuffers( );
}




void resetBall( )
{
	ballSpeed = ballDefaultSpeed;
	ball_x = 0;
	ball_y = 0;
	srand(clock());
	float theta = rand() % 5;
	ball_vx = ballSpeed * cos(PI/4 + theta * PI/8);
	ball_vy = ballSpeed * sin(PI/4 + theta * PI/8);
	passed_paddle = false;
	
	
	// initialize future ball bounds
	future_ball_x = ball_x;
	future_ball_y = ball_y;
	future_ball_vx = ball_vx;
	future_ball_vy = ball_vy;
	
	// calculate the next landing spot! yay!
	findFuturePath( );
	score = 0;
}


void randomizeSnitch( )
{
	float temp = rand() % 1000;
	float leftmost = leftEdge + snitch_radius;
	float rightmost = rightEdge - snitch_radius;
	float usableDistance = fabs( rightmost - leftmost );
	snitch_x = leftmost + (temp / 1000 ) * usableDistance ;
}


void resizeField( float width, float height )
{
	field_width = width;
	field_height = height;
	
	leftEdge = - field_width / 2;
	rightEdge = field_width / 2;
	bottomEdge = - field_height / 2;
	topEdge = field_height / 2;
	
	snitch_y = topEdge - snitch_radius - snitch_radius *.2;
}



// Calculate future landing position
void findFuturePath( )
{
	future_found = false;
	
	while ( !future_found ) {
		
		// increment future_ball's position
		future_ball_x = future_ball_x +  .01 * ballSpeed * future_ball_vx / 1.0;
		future_ball_y = future_ball_y + .01 * ballSpeed * future_ball_vy / 1.0;
		
		
		// update future_ball's bounds
		future_ball_left = future_ball_x - ball_radius;
		future_ball_right = future_ball_x + ball_radius;
		future_ball_bottom = future_ball_y - ball_radius;
		future_ball_top = future_ball_y + ball_radius;
		
		// x
		if ( future_ball_right	> rightEdge ) {
			future_ball_x = rightEdge - (future_ball_right - rightEdge) - ball_radius;
			future_ball_vx = -future_ball_vx;
		}
		else if ( future_ball_left < leftEdge ){
			future_ball_x = leftEdge + (leftEdge - future_ball_left) + ball_radius;
			future_ball_vx = -future_ball_vx;
		}
		// y
		if ( future_ball_top > topEdge ) {
			future_ball_y = topEdge - (future_ball_top - topEdge) - ball_radius;
			future_ball_vy = -future_ball_vy;
		}
		
		else if ( future_ball_bottom < (bottomEdge + paddle_height)){
			future_found = true;
		}		
	}
}



void playBallNote( )
{
	int positionToPlay = 0;
	float stepper = leftEdge;
	while ( ball_x > stepper ) {
		stepper += horizIncrement;
		positionToPlay++;
	}
	int noteToPlay = NoteSequence[(positionToPlay + 1) % 24] + 48;
	dataRef->voicer.noteOn( noteToPlay, 127, 3 );
}


void drawFrame( )
{
	glLineWidth( 3.0 );
	glColor3f( 0, 0, 1 );
 	glBegin( GL_LINE_STRIP );
	glVertex2f( leftEdge , bottomEdge );
	glVertex2f( leftEdge , topEdge );
	glVertex2f( rightEdge , topEdge );
	glVertex2f( rightEdge , bottomEdge );
	glVertex2f( leftEdge , bottomEdge );
    glEnd();


	glColor3f( 1, .5, .5 );
	//char s1[200] = "Gb    Bb    Db    F    Ab    C    Eb    G    Bb    D    F     A     C     E    G     B    D    F#    A    C#    E    G#    B    D#    F#    A#";
	
	//int len, 
	int i, j;
	int pos = 0;
	
	for (i = 0; i < 28; i++) {
		float textSpot = leftEdge + (pos * horizIncrement) + (horizIncrement / 3);
		glRasterPos2f( textSpot, bottomEdge - .2);
		pos++;
		for (j = 0; j < 2; j++) {
			glutBitmapCharacter( GLUT_BITMAP_HELVETICA_18, NoteNames[i][j] );
		}
	}
		
	std::string score_string;
	std::stringstream out;
	out << score;
	score_string = out.str();
	
	glRasterPos2f( rightEdge + .3, topEdge - .3);
	for (i = 0; i < 3; i++) {
		glutBitmapCharacter( GLUT_BITMAP_TIMES_ROMAN_24, score_string[i] );
	}
}



void drawBall( float xPos, float yPos, float rad, bool real, bool snitch )
{	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable( GL_DEPTH_TEST );
	
	GLfloat position[] = { 1.5f, 1.0f, 2.0f, 0.0f };
	glLightfv(GL_LIGHT0, GL_POSITION, position);

	glPushMatrix();
	
	int radius = 5;
	glTranslatef ( xPos, yPos, 0.0);

		
	float opac = 1.0;	

	if ( !real ) { opac = .2; }
	
	
	glColorMaterial ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE ) ;
	glEnable ( GL_COLOR_MATERIAL ) ;
	
	glLineWidth( 1.0 );
	
	if ( snitch ) {
		int param1 = rand() % 4 + 3;
		int param2 = rand() % 4 + 3;
		glPushMatrix();
		snitch_rotation += 1;
		glRotatef( snitch_rotation, 1.0, 1.0, 1.0 );
		glColor4f( 0.0, 1.0, 1.0, opac );
		glutWireSphere ( rad, param1, param2);
		glPopMatrix();
	}
	else {
		glColor4f( 0.6, 1.0, 0.6, opac );
		glutSolidSphere ( rad, 16, 16);

	}

	
	
	glPopMatrix();
	glDisable( GL_DEPTH_TEST );
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHTING);
	
}




void drawPaddle( int position, float opacity )
{
	glLineWidth( 2.0 );
	glColor4f( 1, 0, 0, opacity );
 	glBegin( GL_LINE_STRIP );
		
	currentPositionLeft = leftEdge + (position * horizIncrement);
	currentPositionRight = currentPositionLeft + 3 * horizIncrement;
	
	glVertex2f( currentPositionLeft + paddle_space,		bottomEdge + paddle_space );
	glVertex2f( currentPositionRight - paddle_space,	bottomEdge + paddle_space );
	glVertex2f( currentPositionRight - paddle_space,	bottomEdge + paddle_height );
	glVertex2f( currentPositionLeft + paddle_space,		bottomEdge + paddle_height );
	glVertex2f( currentPositionLeft + paddle_space,		bottomEdge + paddle_space );
    glEnd();
}


void lookupPaddles( )
{
	int pitchClasses[12] = {0,0,0,0,0,0,0,0,0,0,0,0};	
	int chordList [24] = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};

	int paddlesToDraw = 24;
	int noteFound = 0;
	for ( int i = 0; i < 128; i++ ) {
		if ( paddleNotesPlaying[i] != 0 ) {
			noteFound = 1;
			pitchClasses[i % 12] = 1;
		}
	}
	if (noteFound) {
		for (int i = 0; i < 24; i++) {
			for (int j = 0; j < 12; j++) {
				if (pitchClasses[j]) {
					if ((j == NoteSequence[i]) || (j == NoteSequence[i + 1]) || (j == NoteSequence[i + 2]) ) {
						// twiddle
					}
					else {
						if (chordList[i]) {
							chordList[i]= 0;
							paddlesToDraw--;
						}
					}
				}
			}
		}
	}
	else {
		for (int i = 0; i < 24; i++) {
			chordList[i] = 0;
		}
		paddlesToDraw = 0;
	}
	
	for (int i = 0; i < 24; i++) {
		if ( chordList[i] ) {
			drawPaddle( i , .9/paddlesToDraw + .1 );
			if (paddlesToDraw == 1) {
				real_paddle = true;
				paddlePosition = i;
			}
			else {
				real_paddle = false;
			}

		}
	}
}


